package com.deepexi.scd.component.fill.business.report;

import java.util.stream.Stream;
import java.util.function.BinaryOperator;
import com.deepexi.spa.commons.data.Payload;
import com.deepexi.scd.component.fill.domain.dto.CalculateDataFillCalculateItemDTO;
import com.deepexi.scd.calculate.domain.dto.CalculateTemplateItemApiDTO;
import com.deepexi.scd.calculate.domain.query.CalculateTemplateItemApiQuery;
import com.deepexi.scd.calculate.domain.dto.CalculateStandardFieldApiDTO;
import com.deepexi.scd.calculate.domain.query.CalculateStandardFieldApiQuery;
import com.google.common.collect.Maps;
import java.util.Objects;
import com.deepexi.scd.component.fill.domain.query.CalculateDataFillCalculateItemQuery;
import com.deepexi.scd.component.fill.domain.dto.CalculateReportCalculateItemValueDTO;
import java.util.Map;
import com.deepexi.scd.component.fill.domain.dto.CalculateReportDTO;
import java.util.Comparator;
import java.util.Arrays;
import com.deepexi.scd.common.enums.business.CalculateItemValueTypeEnum;
import com.deepexi.scd.component.fill.domain.vo.CalculateLayoutItemValueVO;
import com.deepexi.scd.component.fill.domain.vo.CalculateLayoutItemVO;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import com.deepexi.scd.component.fill.converter.ReportTemplateCovert;
import com.google.common.collect.ImmutableList;
import java.util.Optional;
import com.deepexi.scd.common.enums.BaseExceptionEnum;
import com.deepexi.scd.common.exceptions.BusinessException;
import com.deepexi.scd.component.fill.enums.ResultEnum;
import java.util.function.Function;
import com.deepexi.scd.calculate.domain.dto.CommonDdictMemberApiDTO;
import com.deepexi.scd.calculate.domain.query.CommonDdictMemberApiQuery;
import java.math.BigDecimal;
import com.deepexi.scd.common.extensions.ExNumeric;
import org.apache.commons.collections.CollectionUtils;
import com.deepexi.scd.component.fill.domain.vo.CalculateReportLayoutTotalVO;
import com.deepexi.scd.common.enums.business.CalculateTotalTypeEnum;
import java.util.LinkedList;
import java.util.Collection;
import com.google.api.client.util.Lists;
import com.deepexi.scd.common.enums.business.CalculateReportTableTypeEnum;
import com.deepexi.scd.component.fill.domain.vo.CalculateReportLayoutObjectVO;
import java.util.List;
import com.deepexi.scd.component.fill.domain.vo.CalculateReportLayoutVO;
import com.deepexi.scd.component.fill.remote.calculate.CalculateTemplateItemRemote;
import com.deepexi.scd.component.fill.remote.calculate.CalculateStandardFieldRemote;
import com.deepexi.scd.component.fill.service.CalculateDataFillCalculateItemService;
import com.deepexi.scd.component.fill.service.CalculateDataFillCalculateItemValueService;
import org.springframework.beans.factory.annotation.Autowired;
import com.deepexi.scd.component.fill.service.CommonDdictMemberService;
import org.springframework.stereotype.Component;
import com.deepexi.scd.component.fill.business.CalculateReportLayout;

@Component("06_2")
public class CalculateReportLayout06_2 extends CalculateReportLayout
{
    @Autowired
    private CommonDdictMemberService commonDdictMemberService;
    @Autowired
    private CalculateDataFillCalculateItemValueService calculateDataFillCalculateItemValueService;
    @Autowired
    private CalculateDataFillCalculateItemService calculateDataFillCalculateItemService;
    @Autowired
    private CalculateStandardFieldRemote calculateStandardFieldRemote;
    @Autowired
    private CalculateTemplateItemRemote calculateTemplateItemRemote;
    private static final String INCLUDE_ELECTRIC_HOTTING_TOTAL = "包括净购入电力和热力的隐含CO₂排放";
    private static final String EXCLUDE_ELECTRIC_HOTTING_TOTAL = "不包括净购入电力和热力的隐含CO₂排放";
    private static final String ELECTRIC_TOTAL = "净购入电力隐含的CO₂排放";
    private static final String HOTTING_TOTAL = "净购入热力隐含的CO₂排放";
    public static final String CALCULATE_TABLE = "CALCULATE_TABLE";
    public static final String EMISSION_TOTAL = "EMISSION_TOTAL";
    
    protected void loadTableList(final CalculateReportLayoutVO layout) {
        super.loadTableList(layout);
        super.loadAggregationTargetTable(layout);
        final List<CalculateReportLayoutObjectVO> all = layout.getTableMap().get(CalculateReportTableTypeEnum.ALL.name());
        final List<CalculateReportLayoutObjectVO> objects = Lists.newArrayList();
        objects.addAll(this.getAnnexedTable2(all));
        objects.add(this.getAnnexedTable3(all));
        objects.add(this.getAnnexedTable4(layout.getReportId(), all));
        objects.add(this.getAnnexedTable5(all));
        objects.add(this.getAnnexedTable6(all));
        objects.add(this.getAnnexedTable7(all));
        objects.add(this.getAnnexedTable8(all));
        objects.add(this.getAnnexedTable9(all));
        layout.getTableMap().put("CALCULATE_TABLE", objects);
    }
    
    protected void loadTotalList(final CalculateReportLayoutVO layout) {
        final List<CalculateReportLayout.ReportTotalMapping> reportTotalMappings = new LinkedList<CalculateReportLayout.ReportTotalMapping>();
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("燃料燃烧CO₂排放", "化石燃料燃烧排放量", CalculateTotalTypeEnum.EDGE.name()));
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("火炬燃烧CO₂排放", "火炬燃烧排放量", CalculateTotalTypeEnum.EDGE.name()));
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("CH₄逃逸排放", "CH₄逃逸排放量", CalculateTotalTypeEnum.META.name()));
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("CO₂逃逸排放", "CO₂逃逸排放量", CalculateTotalTypeEnum.META.name()));
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("净购入电力隐含的CO₂排放", "电力购入输出明细排放量", CalculateTotalTypeEnum.TABLE.name()));
        reportTotalMappings.add(new CalculateReportLayout.ReportTotalMapping("净购入热力隐含的CO₂排放", "热力购入输出明细排放量", CalculateTotalTypeEnum.TABLE.name()));
        super.totalMappingList = reportTotalMappings;
        super.loadTotalList(layout, false);
        final List<CalculateReportLayoutTotalVO> total = layout.getTotalMap().get(CalculateTotalTypeEnum.TOTAL.getKey());
        layout.getTotalMap().put(CalculateTotalTypeEnum.TOTAL.getKey(), this.getMainTotal(total));
        this.getEmissionTotal(layout);
    }
    
    private void getEmissionTotal(final CalculateReportLayoutVO layout) {
        final List<CalculateReportLayoutTotalVO> emissionTotals = Lists.newArrayList();
        final List<CalculateReportLayoutTotalVO> co2Totals = layout.getTotalMap().get("二氧化碳（C0₂）");
        if (CollectionUtils.isNotEmpty((Collection)co2Totals)) {
            emissionTotals.addAll(co2Totals);
        }
        final List<CalculateReportLayoutTotalVO> ch4Totals = layout.getTotalMap().get("甲烷CH₄");
        if (CollectionUtils.isNotEmpty((Collection)ch4Totals)) {
            final BigDecimal ch4Gas = this.getCH4GAS();
            ch4Totals.forEach(item -> item.setTotalValueStr(ExNumeric.stepFormatValue(ExNumeric.toBigDecimal(item.getTotalValueStr()).divide(ch4Gas, 10, 4), Integer.valueOf(2))));
            emissionTotals.addAll(ch4Totals);
        }
        layout.getTotalMap().put("EMISSION_TOTAL", emissionTotals);
    }
    
    private BigDecimal getCH4GAS() {
        final CommonDdictMemberApiQuery memberQuery = new CommonDdictMemberApiQuery();
        memberQuery.setDdictType("GAS_TYPE");
        memberQuery.setDdictMember("甲烷CH₄");
        final Optional<String> optional = this.commonDdictMemberService.findList(memberQuery).stream().map(CommonDdictMemberApiDTO::getUdf1).findFirst();
        if (!optional.isPresent()) {
            throw new BusinessException((BaseExceptionEnum)ResultEnum.CANNOT_FIND_CH4_FACTOR);
        }
        return ExNumeric.toBigDecimal((String)optional.get());
    }
    
    private List<CalculateReportLayoutTotalVO> getMainTotal(final List<CalculateReportLayoutTotalVO> totals) {
        final CalculateReportLayoutTotalVO main = new CalculateReportLayoutTotalVO();
        main.setTotalCategory(CalculateTotalTypeEnum.TOTAL.getKey());
        main.setTotalName("包括净购入电力和热力的隐含CO₂排放");
        main.setTotalType(CalculateTotalTypeEnum.MAIN.name());
        final BigDecimal mainTotalValue = totals.stream().map((Function<? super Object, ?>)CalculateReportLayoutTotalVO::getTotalValueStr).map((Function<? super Object, ? extends BigDecimal>)ExNumeric::toBigDecimal).reduce(BigDecimal.ZERO, BigDecimal::add);
        main.setTotalValue(mainTotalValue);
        main.setTotalValueStr(ExNumeric.stepFormatValue(mainTotalValue, 5, 2));
        final CalculateReportLayoutTotalVO excludeElectricAndHotting = new CalculateReportLayoutTotalVO();
        excludeElectricAndHotting.setTotalCategory(CalculateTotalTypeEnum.TOTAL.getKey());
        excludeElectricAndHotting.setTotalName("不包括净购入电力和热力的隐含CO₂排放");
        excludeElectricAndHotting.setTotalType(CalculateTotalTypeEnum.MAIN.name());
        final BigDecimal excludeMainTotalValue = totals.stream().filter(item -> !"净购入电力隐含的CO₂排放".equals(item.getTotalName()) && !"净购入热力隐含的CO₂排放".equals(item.getTotalName())).map((Function<? super Object, ?>)CalculateReportLayoutTotalVO::getTotalValueStr).map((Function<? super Object, ? extends BigDecimal>)ExNumeric::toBigDecimal).reduce(BigDecimal.ZERO, BigDecimal::add);
        excludeElectricAndHotting.setTotalValue(excludeMainTotalValue);
        excludeElectricAndHotting.setTotalValueStr(ExNumeric.stepFormatValue(excludeMainTotalValue, 5, 2));
        totals.add(excludeElectricAndHotting);
        totals.add(main);
        return totals;
    }
    
    private List<CalculateReportLayoutObjectVO> getAnnexedTable2(final List<CalculateReportLayoutObjectVO> all) {
        final List<CalculateReportLayoutObjectVO> tableItems = ReportTemplateCovert.getTableItems((List)all, (List)ImmutableList.of("化石燃料燃烧"), (List)ImmutableList.of("设施燃料燃烧"), (List)ImmutableList.of("重点燃烧设施"), "平均低位发热量", "单位热值含碳量", "含碳量计算结果");
        if (CollectionUtils.isEmpty((Collection)tableItems)) {
            return Lists.newArrayList();
        }
        return ReportTemplateCovert.getTarget((CalculateReportLayoutObjectVO)tableItems.get(0));
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable3(final List<CalculateReportLayoutObjectVO> all) {
        final CalculateReportLayoutObjectVO objectVO = ReportTemplateCovert.getTableItems((List)all, (List)ImmutableList.of("化石燃料燃烧"), (List)ImmutableList.of("设施燃料燃烧"), (List)ImmutableList.of("其他燃烧设施"), "平均低位发热量", "单位热值含碳量", "含碳量计算结果").get(0);
        objectVO.setObjectType("CALCULATE_TABLE");
        objectVO.setObjectName("其他燃烧设施的活动水平和排放因子数据一览表");
        return objectVO;
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable4(final Long reportId, final List<CalculateReportLayoutObjectVO> all) {
        final List<CalculateLayoutItemVO> itemList = Lists.newArrayList();
        final CalculateReportLayoutObjectVO objectVO = new CalculateReportLayoutObjectVO();
        objectVO.setObjectType("CALCULATE_TABLE");
        objectVO.setObjectName("火炬燃烧的活动水平和排放因子数据一览表");
        objectVO.setItemList((List)itemList);
        final CalculateReportLayoutObjectVO tableObject = ReportTemplateCovert.getTableItems((List)all, (List)ImmutableList.of("火炬燃烧"), (List)ImmutableList.of("火炬燃烧CO₂排放"), (List)ImmutableList.of("火炬燃烧CO₂排放"), (String)null, (String)null, (String)null).get(0);
        final List<CalculateLayoutItemValueVO> collect = (List<CalculateLayoutItemValueVO>)tableObject.getItemList().stream().flatMap(item -> item.getItemValueList().stream()).collect(Collectors.toList());
        final BigDecimal consume = collect.stream().filter(itemValue -> "煤层气火炬燃烧量".equals(itemValue.getValueCode())).map(itemValue -> ExNumeric.toBigDecimal(itemValue.getValue())).reduce(BigDecimal.ZERO, BigDecimal::add);
        final List<BigDecimal> carbonConsumeList = collect.stream().filter(itemValue -> "煤层气中非CO₂的总含碳量".equals(itemValue.getValueCode())).map(itemValue -> ExNumeric.toBigDecimal(itemValue.getValue())).collect((Collector<? super Object, ?, List<BigDecimal>>)Collectors.toList());
        final List<BigDecimal> carbonVolumeList = collect.stream().filter(itemValue -> "火炬燃烧的碳氧化率".equals(itemValue.getValueCode())).map(itemValue -> ExNumeric.toBigDecimal(itemValue.getValue())).collect((Collector<? super Object, ?, List<BigDecimal>>)Collectors.toList());
        final BigDecimal carbonConsume = CollectionUtils.isEmpty((Collection)carbonConsumeList) ? BigDecimal.ZERO : carbonConsumeList.stream().reduce(BigDecimal.ZERO, BigDecimal::add).divide(new BigDecimal(carbonConsumeList.size()), 10, 4);
        final BigDecimal carbonVolume = CollectionUtils.isEmpty((Collection)carbonVolumeList) ? BigDecimal.ZERO : carbonVolumeList.stream().reduce(BigDecimal.ZERO, BigDecimal::add).divide(new BigDecimal(carbonVolumeList.size()), 10, 4);
        final CalculateLayoutItemVO consumeItem = new CalculateLayoutItemVO();
        consumeItem.setItemName("煤矿瓦斯的火炬燃烧量（万 Nm³）");
        consumeItem.setTableName("火炬燃烧CO₂排放");
        final CalculateLayoutItemValueVO consumeItemValue = new CalculateLayoutItemValueVO();
        consumeItemValue.setValue(ExNumeric.stepFormatValue(consume, Integer.valueOf(5)));
        consumeItemValue.setValueType(CalculateItemValueTypeEnum.CONSUMPTION.name());
        consumeItem.setItemValueList((List)ImmutableList.of(consumeItemValue));
        itemList.add(consumeItem);
        final CalculateLayoutItemVO carbonVolumeItem = new CalculateLayoutItemVO();
        carbonVolumeItem.setItemName("火炬燃烧的碳氧化率（%）");
        carbonVolumeItem.setTableName("火炬燃烧CO₂排放");
        final CalculateLayoutItemValueVO carbonVolumeItemValue = new CalculateLayoutItemValueVO();
        carbonVolumeItemValue.setValue(ExNumeric.stepFormatValue(carbonVolume, Integer.valueOf(5)));
        carbonVolumeItemValue.setValueType(CalculateItemValueTypeEnum.CONSUMPTION.name());
        carbonVolumeItem.setItemValueList((List)ImmutableList.of(carbonVolumeItemValue));
        itemList.add(carbonVolumeItem);
        final CalculateReportDTO reportDTO = this.calculateReportService.getById(reportId);
        final List<Long> fillIds = Arrays.stream(reportDTO.getFillIds().split(",")).map((Function<? super String, ?>)Long::valueOf).collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
        if (CollectionUtils.isEmpty((Collection)fillIds)) {
            return null;
        }
        final Map<String, Integer> itemSequence = this.getItemSequence(fillIds);
        final List<CalculateReportCalculateItemValueDTO> calculateItemValues = this.calculateDataFillCalculateItemValueService.findReportData((List)fillIds);
        final List<CalculateLayoutItemVO> quoteItemList = calculateItemValues.stream().map(calculateItem -> {
            final CalculateLayoutItemVO item2 = new CalculateLayoutItemVO();
            final List<CalculateLayoutItemValueVO> itemValueList = Lists.newArrayList();
            item2.setItemName(calculateItem.getGasType());
            item2.setTableName("引用表");
            item2.setItemValueList((List)itemValueList);
            final CalculateLayoutItemValueVO carbonItemValue = new CalculateLayoutItemValueVO();
            carbonItemValue.setValue(ExNumeric.stepFormatValue(calculateItem.getCarbonNums(), Integer.valueOf(5)));
            carbonItemValue.setValueCode("碳原子数目");
            carbonItemValue.setValueType(CalculateItemValueTypeEnum.CONSUMPTION.name());
            itemValueList.add(carbonItemValue);
            final CalculateLayoutItemValueVO volumeItemValue = new CalculateLayoutItemValueVO();
            volumeItemValue.setValue(ExNumeric.stepFormatValue(calculateItem.getVolumeConcentration(), Integer.valueOf(5)));
            volumeItemValue.setValueCode("体积浓度");
            carbonItemValue.setValueType(CalculateItemValueTypeEnum.FACTOR.name());
            itemValueList.add(volumeItemValue);
            final CalculateLayoutItemValueVO allCarbonItemValue = new CalculateLayoutItemValueVO();
            allCarbonItemValue.setValue(ExNumeric.stepFormatValue(carbonConsume, Integer.valueOf(5)));
            allCarbonItemValue.setValueCode("总含碳量");
            allCarbonItemValue.setValueType(CalculateItemValueTypeEnum.CONSUMPTION.name());
            itemValueList.add(allCarbonItemValue);
            return item2;
        }).collect((Collector<? super Object, ?, List<CalculateLayoutItemVO>>)Collectors.toList());
        itemList.addAll(quoteItemList.stream().sorted((Comparator<? super Object>)this.getComparator(itemSequence)).collect((Collector<? super Object, ?, Collection<? extends CalculateLayoutItemVO>>)Collectors.toList()));
        return objectVO;
    }
    
    private Map<String, Integer> getItemSequence(final List<Long> fillIds) {
        final CalculateDataFillCalculateItemQuery itemQuery = new CalculateDataFillCalculateItemQuery();
        itemQuery.setFillIdList((List)fillIds);
        final CalculateDataFillCalculateItemDTO one = this.calculateDataFillCalculateItemService.findOne(itemQuery);
        if (Objects.isNull(one)) {
            return (Map<String, Integer>)Maps.newHashMap();
        }
        final CalculateStandardFieldApiQuery fieldQuery = new CalculateStandardFieldApiQuery();
        fieldQuery.setTableId(one.getTableId());
        fieldQuery.setFieldCode("煤层气中非CO₂的总含碳量");
        final Payload<List<CalculateStandardFieldApiDTO>> payload = (Payload<List<CalculateStandardFieldApiDTO>>)this.calculateStandardFieldRemote.findList(fieldQuery);
        if ("0".equals(payload.getCode())) {
            final List<CalculateStandardFieldApiDTO> list = (List<CalculateStandardFieldApiDTO>)payload.getPayload();
            if (CollectionUtils.isNotEmpty((Collection)list)) {
                final CalculateStandardFieldApiDTO field = list.get(0);
                if (Objects.isNull(field) || Objects.isNull(field.getCalculateTemplateTableId())) {
                    return (Map<String, Integer>)Maps.newHashMap();
                }
                final CalculateTemplateItemApiQuery calculateTemplateItemQuery = new CalculateTemplateItemApiQuery();
                calculateTemplateItemQuery.setTableId(field.getCalculateTemplateTableId());
                return (Map)((List)this.calculateTemplateItemRemote.findList(calculateTemplateItemQuery).getPayload()).stream().collect(Collectors.toMap((Function<? super Object, ?>)CalculateTemplateItemApiDTO::getItemName, (Function<? super Object, ?>)CalculateTemplateItemApiDTO::getSequenceNo));
            }
        }
        return (Map<String, Integer>)Maps.newHashMap();
    }
    
    private Comparator<CalculateLayoutItemVO> getComparator(final Map<String, Integer> itemSequenceMap) {
        return (a, b) -> {
            final Integer aSort = Optional.ofNullable(itemSequenceMap.get(a.getItemName())).orElse(Integer.MAX_VALUE);
            final Integer bSort = Optional.ofNullable(itemSequenceMap.get(b.getItemName())).orElse(Integer.MAX_VALUE);
            return aSort - bSort;
        };
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable5(final List<CalculateReportLayoutObjectVO> all) {
        final List<CalculateLayoutItemVO> items = ReportTemplateCovert.getFilterItem((List)all, (List)ImmutableList.of("逃逸排放"), (List)ImmutableList.of("CH₄逃逸"), (List)ImmutableList.of("井工开采CH₄逃逸-已实现瓦斯连续监测", "井工开采CH₄逃逸-未实现瓦斯连续监测"));
        final CalculateReportLayoutObjectVO objectVO = new CalculateReportLayoutObjectVO();
        objectVO.setObjectName("井工开采的 CH₄ 逃逸排放活动水平数据一览表");
        objectVO.setObjectType("CALCULATE_TABLE");
        final List<String> totalColumns = ImmutableList.of("CH₄风排量", "CH₄抽放量", "CH₄的火炬销毁量", "CH₄的回收利用量", "当月CH₄风排量");
        final List<String> nullColumns = ImmutableList.of("当月平均每分钟CH₄风排量", "当月实际工作日数");
        objectVO.setItemList((List)this.aggregateTableItem(items, totalColumns, nullColumns));
        return objectVO;
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable6(final List<CalculateReportLayoutObjectVO> all) {
        final List<CalculateLayoutItemVO> items = ReportTemplateCovert.getFilterItem((List)all, (List)ImmutableList.of("逃逸排放"), (List)ImmutableList.of("CO₂逃逸"), (List)ImmutableList.of("井工开采CO₂逃逸-已实现瓦斯连续监测", "井工开采CO₂逃逸-未实现瓦斯连续监测"));
        final CalculateReportLayoutObjectVO objectVO = new CalculateReportLayoutObjectVO();
        objectVO.setObjectName("井工开采的 CO₂ 逃逸排放活动水平数据一览表");
        objectVO.setObjectType("CALCULATE_TABLE");
        final List<String> totalColumns = ImmutableList.of("CO₂风排量", "CO₂抽放量", "企业回收利用的瓦斯气体中本有的CO₂纯量", "当月CO₂风排量");
        final List<String> nullColumns = ImmutableList.of("当月平均每分钟CO₂风排量", "当月实际工作日数");
        objectVO.setItemList((List)this.aggregateTableItem(items, totalColumns, nullColumns));
        return objectVO;
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable7(final List<CalculateReportLayoutObjectVO> all) {
        final CalculateReportLayoutObjectVO objectVO = new CalculateReportLayoutObjectVO();
        objectVO.setObjectName("露天开采的活动水平和 CH₄ 排放因子数据一览表");
        objectVO.setObjectType("CALCULATE_TABLE");
        final List<CalculateLayoutItemVO> filterItem = ReportTemplateCovert.getFilterItem((List)all, (List)ImmutableList.of("逃逸排放"), (List)ImmutableList.of("CH₄逃逸"), (List)ImmutableList.of("露天开采CH₄逃逸"));
        final List<CalculateLayoutItemValueVO> itemValues = ReportTemplateCovert.valueCodeTotal((List)filterItem);
        final CalculateLayoutItemVO item = new CalculateLayoutItemVO();
        item.setItemName("露天煤矿");
        item.setItemValueList((List)itemValues);
        objectVO.setItemList((List)ImmutableList.of(item));
        return objectVO;
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable8(final List<CalculateReportLayoutObjectVO> all) {
        final CalculateReportLayoutObjectVO objectVO = ReportTemplateCovert.getTableItems((List)all, (List)ImmutableList.of("逃逸排放"), (List)ImmutableList.of("CH₄逃逸"), (List)ImmutableList.of("矿后活动CH₄逃逸"), (String)null, (String)null, (String)null).get(0);
        objectVO.setObjectName("矿后活动的活动水平和 CH₄ 排放因子数据一览表");
        objectVO.setObjectType("CALCULATE_TABLE");
        return objectVO;
    }
    
    private CalculateReportLayoutObjectVO getAnnexedTable9(final List<CalculateReportLayoutObjectVO> all) {
        final CalculateReportLayoutObjectVO objectVO = new CalculateReportLayoutObjectVO();
        objectVO.setObjectName("净购入电力和热力活动水平和排放因子数据一览表");
        objectVO.setObjectType("CALCULATE_TABLE");
        final List<CalculateLayoutItemVO> filterItem = ReportTemplateCovert.getFilterItem((List)all, (List)ImmutableList.of("电力热力"), (List)ImmutableList.of("电力热力明细"), (List)ImmutableList.of("电力购入输出明细", "热力购入输出明细"));
        ReportTemplateCovert.calculate((List)filterItem, "购入量", "输出量", "净购入量", "SUBTRACT");
        objectVO.setItemList((List)filterItem);
        return objectVO;
    }
    
    private List<CalculateLayoutItemVO> aggregateTableItem(final List<CalculateLayoutItemVO> items, final List<String> totalColumns, final List<String> nullColumns) {
        final List<CalculateLayoutItemVO> list = Lists.newArrayList();
        final Map<String, BigDecimal> totalColumnMap = totalColumns.stream().collect(Collectors.toMap(column -> column, item -> BigDecimal.ZERO));
        final List<CalculateLayoutItemValueVO> itemValueList;
        items.stream().collect((Collector<? super Object, ?, Map<Object, List<? super Object>>>)Collectors.groupingBy((Function<? super Object, ?>)CalculateLayoutItemVO::getItemName)).forEach((month, itemList) -> {
            final CalculateLayoutItemVO item = new CalculateLayoutItemVO();
            item.setItemName(month.replace("月", ""));
            final Map<String, List<CalculateLayoutItemValueVO>> valueMap = ReportTemplateCovert.groupByValueCode(itemList);
            final List<CalculateLayoutItemValueVO> itemValueList2 = Lists.newArrayList();
            valueMap.forEach((valueCode, values) -> {
                final CalculateLayoutItemValueVO value = values.get(0);
                final BigDecimal reduce = (BigDecimal)values.stream().map(itemValue -> ExNumeric.toBigDecimal(itemValue.getValue())).reduce(BigDecimal.ZERO, BigDecimal::add);
                value.setValue(ExNumeric.stepFormatValue(reduce, 5, 5));
                final BigDecimal bigDecimal = totalColumnMap.get(valueCode);
                if (Objects.nonNull(bigDecimal)) {
                    totalColumnMap.put(valueCode, bigDecimal.add(ExNumeric.toBigDecimal(value.getValue())));
                }
                itemValueList.add(value);
                return;
            });
            item.setItemValueList((List)itemValueList2);
            list.add(item);
            return;
        });
        final List<CalculateLayoutItemVO> result = list.stream().sorted(Comparator.comparing((Function<? super Object, ? extends Comparable>)CalculateLayoutItemVO::getItemName)).collect((Collector<? super Object, ?, List<CalculateLayoutItemVO>>)Collectors.toList());
        final CalculateLayoutItemVO itemVO = new CalculateLayoutItemVO();
        itemVO.setItemName("合计：");
        itemValueList = Lists.newArrayList();
        itemVO.setItemValueList((List)itemValueList);
        totalColumnMap.forEach((valueCode, value) -> {
            final CalculateLayoutItemValueVO calculateLayoutItemValueVO = new CalculateLayoutItemValueVO();
            calculateLayoutItemValueVO.setValueCode(valueCode);
            calculateLayoutItemValueVO.setValue(ExNumeric.stepFormatValue(value, 5, 5));
            itemValueList.add(calculateLayoutItemValueVO);
            return;
        });
        if (CollectionUtils.isNotEmpty((Collection)nullColumns)) {
            final List<CalculateLayoutItemValueVO> collect = nullColumns.stream().map(valueCode -> {
                final CalculateLayoutItemValueVO calculateLayoutItemValueVO2 = new CalculateLayoutItemValueVO();
                calculateLayoutItemValueVO2.setValueCode(valueCode);
                calculateLayoutItemValueVO2.setValue("/");
                return calculateLayoutItemValueVO2;
            }).collect((Collector<? super Object, ?, List<CalculateLayoutItemValueVO>>)Collectors.toList());
            itemValueList.addAll(collect);
        }
        result.add(itemVO);
        return result;
    }
}
